import nGeneFastICA
import requests
import numpy as np
import matplotlib.pyplot as plt
import certifi
import urllib3
from urllib3.exceptions import InsecureRequestWarning
#########################
from pyscript import display
from pyweb import pydom
from pyodide.http import open_url
from js import console
##########################
from js import document, fetch, URL

########################################################################################
def log(message):
    print(message)        
    console.log(message)  

########################################################################################
def fetch_wav_and_convert_to_numpy(url):
    response = requests.get(url, verify=certifi.where())
    if response.status_code != 200:
        print("Failed to download the file:", url)
        return None
    wav_bytes = response.content
    audio_data = np.frombuffer(wav_bytes[44:], dtype=np.int16)
    normalized_audio = audio_data / np.max(np.abs(audio_data))
    return normalized_audio

def plot_waveforms(signals, sample_rate, titles, colors):
    total_num_plots = len(signals)
    plt.figure(figsize=(6.5, 3 * total_num_plots), dpi=100)

    for i, (audio_array, title, color) in enumerate(zip(signals, titles, colors), start=1):
        time_axis = np.linspace(0, len(audio_array) / sample_rate, num=len(audio_array))
        plt.subplot(total_num_plots, 1, i)
        plt.plot(time_axis, audio_array, label=title, color=color)
        plt.title(title)
        plt.xlabel("Time (seconds)")
        plt.ylabel("Amplitude (normalized)")
        plt.legend()
        plt.tight_layout()

    plt.show()
    display(plt, target="output_ica")    
    display(plt.subplots(), target="output_ica")

############################################################################################    
def run_ica(event):
    urllib3.disable_warnings(InsecureRequestWarning)
        
    # URLs corrected to .wav
    # heart_sound_url = "https://raw.githubusercontent.com/nGene-org/SVM/main/heart_sound.wav"
    # lung_sound_url = "https://raw.githubusercontent.com/nGene-org/SVM/main/lung_sound.wav"
    heart_sound_url = pydom["input#heartSound-url"][0].value
    lung_sound_url = pydom["input#lungSound-url"][0].value
    
    log(f"Heart Sound URL: {heart_sound_url}")
    log(f"Lung  Sound URL: {lung_sound_url}\n")
    
    # Fetch and convert audio data to numpy arrays
    heart_sound = fetch_wav_and_convert_to_numpy(heart_sound_url)
    lung_sound = fetch_wav_and_convert_to_numpy(lung_sound_url)

    # Assuming a common sample rate for audio files
    sr = 44100  # Sample rate in Hz

    # Stack both signals for ICA
    S = np.vstack((heart_sound, lung_sound))

    # Mix signals
    A = np.array([[1, 1], [0.5, 2]])
    X = A @ S

    # Perform FastICA
    fastICA = nGeneFastICA.FastICA(num_sources=2)
    components = fastICA.decompose_mixtures(X)

    # Log the ICA components
    log(f"ICA Components:\nComponent 1: {components[0, :]}\nComponent 2: {components[1, :]}\n")
            
    # Preparing data for plotting
    signals = [heart_sound, lung_sound, X[0, :], X[1, :], components[0, :], components[1, :]]
    titles = ["Original Heart Sound Waveform", "Original Lung Sound Waveform",
              "Mixed Signal 1", "Mixed Signal 2",
              "Separated Signal 1", "Separated Signal 2"]
    colors = ['red', 'blue', 'green', 'orange', 'purple', 'cyan']

    # Plotting
    plot_waveforms(signals, sr, titles, colors)

    # Log the plotting result
    log("Waveforms have been plotted successfully.\n")
            
############################################################################################
log(f"Now Initialized.\n")
